/* Copyright 2004, 2005 Acegi Technology Pty Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.rfxsoft.justrfx.acegi;

import org.acegisecurity.acl.AclEntry;
import org.acegisecurity.acl.AclManager;

import org.springframework.beans.factory.InitializingBean;

import org.springframework.util.Assert;

import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.AbstractController;

import java.io.IOException;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * Controller for "administer" index page.
 *
 * @author Ben Alex
 * @version $Id: AdminPermissionController.java,v 1.4 2006/12/05 07:47:35 k1park Exp $
 */
public class AdminPermissionController extends AbstractController implements /*Controller,*/ InitializingBean {
    //~ Instance fields ========================================================

    private AclManager aclManager;
    private ContactManager contactManager;

    //~ Methods ================================================================

    public void setAclManager(AclManager aclManager) {
        this.aclManager = aclManager;
    }

    public AclManager getAclManager() {
        return aclManager;
    }

    public void setContactManager(ContactManager contact) {
        this.contactManager = contact;
    }

    public ContactManager getContactManager() {
        return contactManager;
    }

    public void afterPropertiesSet() throws Exception {
        Assert.notNull(contactManager,
            "A ContactManager implementation is required");
        Assert.notNull(aclManager, "An aclManager implementation is required");
    }

    public ModelAndView handleRequestInternal(HttpServletRequest request,
        HttpServletResponse response) throws ServletException, IOException {
        int id = ServletRequestUtils.getRequiredIntParameter(request, "contactId");

        Contact contact = contactManager.getById(new Long(id));
        AclEntry[] acls = aclManager.getAcls(contact);

        Map<String, Object> model = new HashMap<String, Object>();
        model.put("contact", contact);
        model.put("acls", acls);

        return new ModelAndView("adminPermission", "model", model);
    }
}
