package com.rfxsoft.justrfx.business.c_prjt;

import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.springframework.beans.factory.BeanCreationException;
import org.springframework.beans.factory.InitializingBean;

import com.rfxsoft.justrfx.domain.ComBst;
import com.rfxsoft.justrfx.domain.Project;
import com.rfxsoft.justrfx.domain.PjtTsk;
import com.rfxsoft.justrfx.domain.TskMap;

public class DefaultPrjtManager extends AbstractDaoBasedPrjtManager implements
InitializingBean {
	
	private static final String CONSTANT = "Must set projectDao on DefaultPrjtManager"; //$NON-NLS-1$


	public void afterPropertiesSet() throws Exception {
		if (projectDao == null)
			throw new BeanCreationException(CONSTANT);
		if (pjttskDao == null)
			throw new BeanCreationException("Must set pjttskDao on DefaultPrjtManager");
		if (combstDao == null)
			throw new BeanCreationException("Must set combstDao on DefaultPrjtManager");
		if (tskmapDao == null)
			throw new BeanCreationException("Must set tskmapDao on DefaultPrjtManager");

	}
	
	
	public boolean deleteProject(int unique_id) {
		projectDao.delete(unique_id);
		return true;
	}
	public boolean complishProject(int unique_id) {
		projectDao.complish(unique_id);
		return true;
	}
	public List getDynamic(Map condition) {
		return projectDao.getDynamic(condition);
	}
	
	public Project getProject(String pj_code) {
		return projectDao.getByCode(pj_code);
	}
	
	public Project getProjectByUniqueId(int unique_id) {
		return projectDao.getById(unique_id);
	}
	
	public List getProjectList() {
		return projectDao.getAll();
	}

	public void addProject(String creator, Project project) {
		projectDao.insert(creator, project);
	}

	public List getProjectBom(String pj_code) {

		Map<String, String> map = new HashMap<String, String>();
		
		map.put("pj_code",	pj_code);
		
		return projectDao.getDynamic(map);

	}

	public boolean modifyProject(Project project) {
		
		if(project.getOrder_com_unique() == null) {
			String wa_code = project.getWa_code();
			if(wa_code == null)
				return false;
			ComBst combst =  combstDao.getByCode(wa_code);
			int com_unique = combst.getUnique_id();
			project.setOrder_com_unique(com_unique);
		}
		projectDao.update(project);
		return true;
	}
	
	public void setCurrentProject(String pj_code) {
		// TODO Auto-generated method stub

	}
	
	public List getTaskList( int pj_unique) {
		return pjttskDao.getByParent(pj_unique);
	}
	
	public List getTaskList( String pj_code) {
		
		Project project = projectDao.getByCode(pj_code);
		
		if(project==null)
			return null;
		
		return pjttskDao.getByParent(project.getUnique_id());
		
	}

	public Object addTask(String creator, PjtTsk task) {
		return pjttskDao.insert(creator, task);

	}

	public void modifyTask(PjtTsk task) {
		pjttskDao.update(task);

	}

	public void deleteTask(Integer task_id) {
		pjttskDao.delete(task_id);

	}

	public List getProjectList(Map condition) {
		return projectDao.getDynamic(condition);
	}


	public void setPurchaseCost(String pj_code, Double amount) {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("purchase_cost", amount);
		map.put("pj_code", pj_code);
		
		projectDao.updatePrchAmount(map);
		
	}


	public TskMap getTaskMapToday(int task_uid, String map_date) {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("task_uid", task_uid);
		map.put("map_date", map_date);
		//  List ϳ Ѵ.
		
		TskMap tskmap = null;
		
		List list = tskmapDao.select_cond("tskmap", map);
		if(list!=null && list.size()>0) {
			Iterator i=list.iterator();
			tskmap = (TskMap)i.next();
		}

		return tskmap;
	}
	
	public void addTaskMap(String creator, TskMap tskmap) {
		tskmapDao.insert(creator, tskmap);

	}
	
	public void addTaskMap(String creator, String map_date, Integer task_uid,
			Double resource_mh) {
		
		//update pjttsk startdate 
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("changer", creator);
		map.put("start_date", new Date());
		map.put("unique_id", task_uid);
		
		pjttskDao.updateSateOpen(map);
		
		TskMap tskmap = new TskMap();
		
		tskmap.setMap_date(map_date);
		tskmap.setTask_uid(task_uid);
		tskmap.setResource_mh(resource_mh);
		tskmap.setChange_date(new Date());
		tskmap.setChanger(creator);
		
		tskmapDao.insert(creator, tskmap);

	}

	public boolean modifyTaskMap(String changer, Integer tskmap_uid, Double resource_mh) {

		TskMap tskmap = tskmapDao.getById(tskmap_uid);
		
		if(tskmap==null)
			return false;
		
		tskmap.setResource_mh(resource_mh);
		tskmap.setChanger(changer);
		tskmap.setChange_date(new Date());
		
		tskmapDao.update(tskmap);
		
		return true;
	}
	
	public void doCloseTaskIter(String changer, Vector pjttsk_uids) {
		
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("changer", changer);
		map.put("change_date", new Date());
		map.put("end_date", new Date());
		map.put("is_opened", "Y");
		map.put("is_closed", "Y");
		map.put("unique_id_list", pjttsk_uids);
		
		pjttskDao.updateByUidIter(map);
	}

	public void updateResourceMh(Integer task_uid) {
		
		pjttskDao.updateResourceMh(task_uid);
	}


	public void updateProjectHumanPlan(Integer pj_uid, Double human_plan) {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("unique_id", pj_uid);
		map.put("human_plan", human_plan);
		projectDao.updateCondByUid(map);
	}


	public void updateProjectHumanCost(Integer pj_uid, Double human_cost) {
		Map<String, Object> map = new HashMap<String, Object>();
		map.put("unique_id", pj_uid);
		map.put("human_cost", human_cost);
		projectDao.updateCondByUid(map);
	}


	public void modifyTask(Map cond) {
		pjttskDao.updateByUid(cond);
		
	}


	public PjtTsk getTaskByUid(Integer unique_id) {
		
		return pjttskDao.getById(unique_id);
	}
	
}





