package com.rfxsoft.justrfx.document;

import java.awt.Color;
import java.io.IOException;
import java.net.MalformedURLException;


import org.springframework.beans.factory.InitializingBean;

import com.lowagie.text.BadElementException;
import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.ExceptionConverter;
import com.lowagie.text.Font;
import com.lowagie.text.Image;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfPTable;
import com.lowagie.text.pdf.PdfPageEventHelper;
import com.lowagie.text.pdf.PdfSpotColor;
import com.lowagie.text.pdf.PdfWriter;
import com.lowagie.text.pdf.SpotColor;

public abstract class AbstractPdfDocuLayout extends PdfPageEventHelper implements InitializingBean{

	protected Document doc;
	protected PdfWriter writer;
	protected int intProgress;
	protected String docuNo;
	protected String strLogoImagePath;
	protected String strSignImagePath;
	
	protected Image headerImage;
	protected PdfPTable table;
	protected BaseFont helv;

    
	protected BaseFont bfKorea;
	protected BaseFont bf;
	protected Font FontKorea28;
	protected Font FontKorea12;
	protected Font FontKorea12b;
	protected Font FontKorea16;
	protected Font FontKorea10;
	protected Font FontKorea10s;
	protected Font FontKorea10b;
	protected Font FontKorea8;
	protected Font FontKorea6;
	protected Font FontKorea8s;
	protected Font FixedEng8;
    
	protected Color clrBlack;
	protected Color clrWhite;
	protected Color clrCBack;
	protected Color clrDisp;
	protected Color clrPurple;
	
	protected PdfContentByte cb;
	
	protected Image bar2Dimg;
	protected Image logo;
	protected Image sign;
	
	protected String strBarCode;	
	protected final String strZero = "0";

//	protected PdfPTable hTable;
//	protected PdfPTable gTable;
//	protected PdfPCell cell;
	
	protected float global_width[] = {1.0f};
	
    public AbstractPdfDocuLayout(PdfWriter writer, Document doc, int progress,
    		String strCode, String logo_img_path, String sign_img_path) {
		super();
		// TODO Auto-generated constructor stub
		this.doc = doc;
		this.writer = writer;
		this.intProgress = progress;
		this.docuNo	= strCode;
		this.strLogoImagePath = logo_img_path;
		this.strSignImagePath = sign_img_path;
		Init();
	}
    
    public void DoBuildProcess() {
    	GetFileContent();
        CreateHeaderTable();
        //onStartPage( writer,  doc);
        DoWork();
    }
    
    public void Init() {
   		String pdfEncoding="UniKS-UCS2-H";
		String fontName="HYGoThic-Medium";

        PdfSpotColor spc_font = new PdfSpotColor("PANTONE 147", 0.9f, new Color(0, 0, 0));
    
        try {
			bfKorea = BaseFont.createFont(fontName, pdfEncoding,  BaseFont.NOT_EMBEDDED);
			bf = BaseFont.createFont("Helvetica", "winansi", false, false, null, null);
			 bf.setForceWidthsOutput(true);
        } catch (DocumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
        FontKorea28 = new Font(bfKorea, 28, Font.NORMAL, new SpotColor(spc_font));
        FontKorea12 = new Font(bfKorea, 12, Font.NORMAL, new SpotColor(spc_font));
        FontKorea12b = new Font(bfKorea, 12, Font.BOLD, new SpotColor(spc_font));
        FontKorea16 = new Font(bfKorea, 16, Font.NORMAL, new SpotColor(spc_font));
        FontKorea10 = new Font(bfKorea, 10, Font.NORMAL, new SpotColor(spc_font));
        FontKorea10s = new Font(bfKorea, 10, Font.STRIKETHRU, new SpotColor(spc_font));
        FontKorea10b = new Font(bfKorea, 10, Font.BOLD, new SpotColor(spc_font));
        FontKorea8 = new Font(bfKorea, 8, Font.NORMAL, new SpotColor(spc_font));
        FontKorea8s = new Font(bfKorea, 8, Font.STRIKETHRU, new SpotColor(spc_font));
        FontKorea6 = new Font(bfKorea, 6, Font.NORMAL, new SpotColor(spc_font));
        FixedEng8 = new Font(bf, 8, Font.NORMAL, new SpotColor(spc_font));
        
        clrBlack = new Color(0x00, 0x00, 0x00);
        clrWhite = new Color(255, 255, 255);
        clrCBack = new Color(0xCC, 0x99, 0xFF);
        clrDisp = new Color(0xF5, 0xEB, 0xFF);
        clrPurple = new Color(0x80, 0x00, 0x80);
        
        cb = writer.getDirectContent();
        
        try {
			logo = Image.getInstance(strLogoImagePath);
			sign = Image.getInstance(strSignImagePath);
			
		} catch (BadElementException e2) {
			e2.printStackTrace();
		} catch (MalformedURLException e2) {
			e2.printStackTrace();
		} catch (IOException e2) {
			e2.printStackTrace();
		}
        float height = logo.plainHeight();
        logo.scalePercent(100*38/height);
        logo.setAlignment(Image.LEFT);
        
        
		byte gradient[] = new byte[256];
		height = sign.plainHeight();
		sign.scalePercent(100*56/height);
		sign.setAlignment(Image.RIGHT);
		sign.setAbsolutePosition(594-56, 841-115);
        for (int k = 0; k < 256; ++k)
            gradient[k] = (byte)200;//(byte)k;
		Image smask=null;
		try {
			smask = Image.getInstance(60, 1, 1, 8, gradient);
		} catch (BadElementException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			smask.makeMask();
		} catch (DocumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			sign.setImageMask(smask);
		} catch (DocumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
       
        
    }
    
    public String getDocuNo() {
		return docuNo;
	}
    
    /**
     * @see com.lowagie.text.pdf.PdfPageEventHelper#onOpenDocument(com.lowagie.text.pdf.PdfWriter, com.lowagie.text.Document)
     */
    public void onOpenDocument(PdfWriter writer, Document document) {
        try {

        }
        catch(Exception e) {
            throw new ExceptionConverter(e);
        }
    }    

    
    /**
     * @see com.lowagie.text.pdf.PdfPageEventHelper#onStartPage(com.lowagie.text.pdf.PdfWriter, com.lowagie.text.Document)
     */
    public void onStartPage(PdfWriter writer, Document document) {

    	if( document.isOpen() == false)
        	return;
        PdfContentByte cb = writer.getDirectContentUnder();
        cb.saveState();
	
    }
    
    /**
     * @see com.lowagie.text.pdf.PdfPageEventHelper#onCloseDocument(com.lowagie.text.pdf.PdfWriter, com.lowagie.text.Document)
     */
    public void onCloseDocument(PdfWriter writer, Document document) {

    }

	protected abstract void GetFileContent();
    protected abstract void CreateHeaderTable();
    protected abstract void DoWork();
}
