package com.rfxsoft.justrfx.document;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import com.lowagie.text.Chunk;
import com.lowagie.text.Document;
import com.lowagie.text.Element;
import com.lowagie.text.Image;
import com.lowagie.text.Paragraph;
import com.lowagie.text.Phrase;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.Barcode;
import com.lowagie.text.pdf.BarcodeEAN;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfPCell;
import com.lowagie.text.pdf.PdfPTable;
import com.lowagie.text.pdf.PdfWriter;
import com.rfxsoft.justrfx.domain.PartLine;
import com.rfxsoft.justrfx.domain.Project;
import com.rfxsoft.justrfx.domain.RegisterdMasterCompany;
import com.rfxsoft.justrfx.domain.SrcAhd;
import com.rfxsoft.justrfx.domain.SystemInfo;

public class PdfDocuPl extends AbstractPdfDocuLayout {

	private RegisterdMasterCompany masterCompany;
	private SystemInfo	systemInfo;
	private Project		project;
	private SrcAhd		assembly;
	private	List		listPL;
	
//	private SimpleDateFormat dateFormat 		= new SimpleDateFormat("yyyy/MM/dd");
	private SimpleDateFormat dateFormatDetail 	= new SimpleDateFormat("yyyy/MM/dd, H:mm:ss");
//	private SimpleDateFormat dateFormatMedium 	= new SimpleDateFormat("MM/dd,H:mm:ss");

	private DecimalFormat 	decimalFormat = new DecimalFormat("#,##0");
	
	//--------------------- PDF Contents -------------------
	final static String strTitle = "Part List(P/L)";
	
	private String project_info;
	private String assembly_info;
	
	protected PdfPTable hTable;
	protected PdfPTable gTable;
	protected PdfPCell cell;
	
	public PdfDocuPl(PdfWriter writer, Document doc, int progress, String strCode, String logo_img_path, String sign_img_path, 
			RegisterdMasterCompany masterCompany,
			SystemInfo	systemInfo,
			Project		project,
			SrcAhd		assembly,
			List	listPL) {
		
			super(writer, doc, progress, strCode, logo_img_path, sign_img_path);
			this.setMasterCompany(masterCompany);
			this.setSystemInfo(systemInfo);
			this.setProject(project);
			this.setAssembly(assembly);
			this.setListPL(listPL);
	}

	@Override
	protected void GetFileContent() {
		project_info = "[" + project.getPj_code() + "] " +
			project.getPj_name();
		
		assembly_info = "[" + assembly.getItem_code().substring(4,8) + "] " +
			assembly.getItem_name();

		//ѱ ڵ
		strBarCode = "880";//1 ū 
		
		String strCodeFormatted= assembly.getUnique_id().toString();
		while(strCodeFormatted.length()<10)
			strCodeFormatted = "0" + strCodeFormatted;
		
		if(strCodeFormatted.length() > 10 )
			strCodeFormatted = strCodeFormatted.substring(0,10);
		
		//880 ڵ
		strBarCode = strBarCode + strCodeFormatted;

	}

	@Override
	protected void CreateHeaderTable() {
		float[] widths = 			{0.20f, 0.60f, 0.20f};
		float[] inh_widths = 		{1.0f};
		//float[] inh3_widths = 	{0.25f, 0.75f};
		
		hTable = new PdfPTable(widths);
		hTable.getDefaultCell().setBorder(Rectangle.NO_BORDER);
		
		PdfPTable inh1 = new PdfPTable(inh_widths);
		inh1.getDefaultCell().setBorder(Rectangle.NO_BORDER);
		inh1.addCell(new Phrase(new Chunk(logo, 0, 0)));
		
		PdfPTable inh2 = new PdfPTable(inh_widths);
		inh2.getDefaultCell().setBorder(Rectangle.NO_BORDER);
		Chunk chTitle = new Chunk(strTitle, FontKorea28);
		chTitle.setBackground(clrDisp);
		chTitle.setUnderline(clrPurple, 2.0f, 0.0f, 13.0f, 0.0f, PdfContentByte.LINE_CAP_BUTT);
		chTitle.setTextRenderMode(PdfContentByte.TEXT_RENDER_MODE_FILL_STROKE, 0.4f, clrBlack);
		
		inh2.getDefaultCell().setVerticalAlignment(Element.ALIGN_MIDDLE);
		inh2.getDefaultCell().setHorizontalAlignment(Element.ALIGN_CENTER);
		inh2.addCell(new Phrase(chTitle) );
		
		cb = writer.getDirectContent();
		BarcodeEAN codeEAN = new BarcodeEAN();
		codeEAN.setCodeType(Barcode.EAN13);
		codeEAN.setCode(strBarCode);
		Image imageEAN = codeEAN.createImageWithBarcode(cb, null, null);


		PdfPTable inh3 = new PdfPTable(inh_widths);
		inh3.getDefaultCell().setBorder(Rectangle.NO_BORDER);
		inh3.addCell(new Phrase(new Chunk(imageEAN, 0, 0)));

		hTable.getDefaultCell().setVerticalAlignment(Element.ALIGN_MIDDLE);
		hTable.getDefaultCell().setHorizontalAlignment(Element.ALIGN_LEFT);
		hTable.addCell( inh1 );
		hTable.getDefaultCell().setHorizontalAlignment(Element.ALIGN_CENTER);
		hTable.addCell( inh2 );
		hTable.getDefaultCell().setHorizontalAlignment(Element.ALIGN_RIGHT);
		hTable.addCell( inh3 );
	
		float global_width[] = {1.0f};

		gTable = new PdfPTable(global_width);
		gTable.getDefaultCell().setVerticalAlignment(Element.ALIGN_MIDDLE);
		gTable.setHorizontalAlignment(Element.ALIGN_CENTER);
   
		AddHeaderTable( gTable );

	}

	private void AddHeaderTable(PdfPTable gTable) {
		
		float cus_ad_width[] = {0.1f, 0.4f, 0.1f, 0.4f};
		
		PdfPTable table = new PdfPTable(cus_ad_width);
		
		table.getDefaultCell().setVerticalAlignment(Element.ALIGN_MIDDLE);

		table.getDefaultCell().setBackgroundColor(clrCBack);
		table.getDefaultCell().setHorizontalAlignment(Element.ALIGN_CENTER);
		table.addCell(new Phrase("Ʈ", FontKorea10b) );
		table.getDefaultCell().setBackgroundColor(clrWhite);
		table.getDefaultCell().setHorizontalAlignment(Element.ALIGN_LEFT);
		
		table.addCell(new Phrase(project_info, FontKorea10) );
		
		table.getDefaultCell().setBackgroundColor(clrCBack);
		table.getDefaultCell().setHorizontalAlignment(Element.ALIGN_CENTER);
		table.addCell(new Phrase("Assembly", FontKorea10b) );
		table.getDefaultCell().setBackgroundColor(clrWhite);
		table.getDefaultCell().setHorizontalAlignment(Element.ALIGN_LEFT);
		table.addCell(new Phrase(assembly_info, FontKorea10) );
		
		gTable.addCell( table );
	}
	
	
	@Override
	protected void DoWork() {
		
        try {
    		float detail_width[] = {
    				0.04f,
					0.05f,
					0.15f,
					0.24f,
					0.17f,
					0.06f,
					0.09f,
					0.09f,
					0.11f};
    		
        	this.writer.setPageEvent(this);
        	
			table = new PdfPTable(detail_width);
			table.getDefaultCell().setHorizontalAlignment(Element.ALIGN_CENTER);
			table.getDefaultCell().setVerticalAlignment(Element.ALIGN_MIDDLE);
			table.getDefaultCell().setBackgroundColor(clrDisp);
			
			table.addCell(new Phrase("No", FontKorea8) );
			table.addCell(new Phrase("ڵ", FontKorea10) );
			table.addCell(new Phrase("ǰ", FontKorea10) );
			table.addCell(new Phrase("԰", FontKorea10) );
			table.addCell(new Phrase(" / ó / Ŀ", FontKorea8) );
			//table.addCell(new Phrase("ó", FontKorea10) );
			table.addCell(new Phrase("", FontKorea10) );
    		table.addCell(new Phrase("ܰ", FontKorea10) );
    		table.addCell(new Phrase("ݾ", FontKorea10) );
			table.addCell(new Phrase("ȣ", FontKorea8) );
			
    		Double quan_sum = 0.0;
    		Double price_sum = 0.0;
			int serial_no = 1;
			for(Iterator i=listPL.iterator(); i.hasNext(); serial_no++) {
				
				PartLine partline = (PartLine)i.next();
				
				//Origin 
				if(partline.getIs_origin().compareToIgnoreCase("Y")==0) {
				
					String strNo = Integer.toString(serial_no);
					String pl_no = partline.getPl_no();
					String item_code = partline.getItem_code_dash();
					String item_name = partline.getItem_name();
					String specification = partline.getSpecification();
					String maker_name = partline.getMaker_name();
					String material = partline.getModel_no();
					String treatment = partline.getDescription();
					
	    			String unit_price = Double2String(partline.getSales_price() );
	    			String total_price = Double2String(partline.getSales_price()* partline.getQuan());
	
					
					String bm_qty = Double2String(partline.getBm_quan() );
					String class_code = partline.getClass_code();
					
	    			quan_sum = quan_sum + partline.getBm_quan();
	    			price_sum = price_sum + partline.getSales_price()* partline.getBm_quan();
					
					String material_maker = "";
					
					if(material!=null)
						material_maker = material;
					
					if(treatment!=null && treatment.length()>0) {
						if(material_maker.length()>0)
							material_maker = material_maker + " / " + treatment;
						else
							material_maker = material_maker + treatment;
					}
	
					
					if(maker_name!=null && maker_name.length()>0) {
						if(material_maker.length()>0)
							material_maker = material_maker + " / " + maker_name;
						else
							material_maker = material_maker + maker_name;
					}
	
					String mat_type = "";
					
					if(class_code != null && class_code.length()>0) {
						mat_type = class_code.substring(0,1);
						if(mat_type.compareToIgnoreCase("O")==0)
							mat_type = "";
						else if(mat_type.compareToIgnoreCase("S")==0)
							mat_type = "ǥ";
						else if(mat_type.compareToIgnoreCase("M")==0)
							mat_type = "Ŀ";
						
					}
					
					table.getDefaultCell().setBackgroundColor(clrWhite);
					table.getDefaultCell().setHorizontalAlignment(Element.ALIGN_CENTER);
					table.addCell(new Phrase( strNo, FixedEng8) );
					table.addCell(new Phrase( pl_no, FixedEng8) );
					table.getDefaultCell().setHorizontalAlignment(Element.ALIGN_LEFT);
					table.addCell(new Phrase( item_name, FontKorea8) );
					table.addCell(new Phrase( specification, FontKorea8) );
					table.addCell(new Phrase( material_maker, FontKorea8) );
					//table.addCell(new Phrase( treatment, FontKorea8) );
		   			table.getDefaultCell().setHorizontalAlignment(Element.ALIGN_RIGHT);
					table.addCell(new Phrase( bm_qty, FontKorea8) );
		   			table.getDefaultCell().setHorizontalAlignment(Element.ALIGN_RIGHT);
					table.addCell(new Phrase( unit_price, FontKorea8) );
		   			table.getDefaultCell().setHorizontalAlignment(Element.ALIGN_RIGHT);
					table.addCell(new Phrase( total_price, FontKorea8) );
					table.getDefaultCell().setHorizontalAlignment(Element.ALIGN_CENTER);
					table.addCell(new Phrase(item_code, FontKorea6) );
				}
			}
			
    		cell = new PdfPCell(new Phrase("   ", FontKorea10) );
    		cell.setHorizontalAlignment(Element.ALIGN_RIGHT);
    		cell.setVerticalAlignment(Element.ALIGN_MIDDLE);
    		cell.setBackgroundColor(clrDisp);
    		cell.setColspan(5);
    		table.addCell(cell );
    		
    		table.getDefaultCell().setHorizontalAlignment(Element.ALIGN_RIGHT);
    		table.addCell(new Phrase(Double2String(quan_sum), FontKorea8) );
    		table.getDefaultCell().setHorizontalAlignment(Element.ALIGN_CENTER);
    		table.addCell(new Phrase("-", FontKorea8) );
    		table.getDefaultCell().setHorizontalAlignment(Element.ALIGN_RIGHT);
    		table.addCell(new Phrase(Double2String(price_sum), FontKorea8) );
    		table.getDefaultCell().setHorizontalAlignment(Element.ALIGN_CENTER);
    		table.addCell(new Phrase("-", FontKorea8) );

    		
    		table.setTotalWidth(594-20);
    		table.setLockedWidth(true);
    		doc.add(table);
		
			Chunk chEnd = new Chunk("-----   -----", FontKorea12);
			chEnd.setBackground(clrDisp);
			Paragraph end = new Paragraph();
	        end.setAlignment(PdfContentByte.ALIGN_CENTER);
	        end.add(chEnd);
			doc.add(end);
	
			cb.beginText();
			cb.setFontAndSize(bfKorea, 8);
	
			String text = "This is End:";
	 		cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, text, 594-10-30, 5, 0);
			cb.endText();
			
			intProgress = writer.getPageNumber();
	
	    }
	    catch ( Exception e ) {
	        e.printStackTrace();
	    }

	}

    public void onEndPage(PdfWriter writer, Document document) {
        PdfContentByte cb = writer.getDirectContent();
        cb.saveState();
        
		hTable.setTotalWidth(document.right() - document.left());
		hTable.setLockedWidth(true);
		hTable.writeSelectedRows(0, -1, 5, 831, cb);
		hTable.setHeaderRows(2);
        
		//		ܺ 輱
		cb.moveTo(10, 841-85+25);
		cb.lineTo(594-10, 841-85+25);
		cb.stroke();
		cb.saveState();
		
		gTable.setTotalWidth(document.right() - document.left() -10);
		gTable.setLockedWidth(true);
		gTable.writeSelectedRows(0, -1, 10, 831-90+9+25, cb);

		//
		Date today = new Date();
		cb.beginText();
		cb.setFontAndSize(bfKorea, 8);
		//cb.setRGBColorFill(0x80, 0x00, 0x80);
		String text = "μϽ:";
		text = text + dateFormatDetail.format(today); // + ", Powered by justRFX(tm) / www.justrfx.com";
	  
		cb.showTextAligned(PdfContentByte.ALIGN_LEFT, text, 10, 5, 0);
		text = "Page " + writer.getPageNumber();
 		cb.showTextAligned(PdfContentByte.ALIGN_RIGHT, text, 594-10, 5, 0);
		cb.endText();
		
		//ϴܺ 輱
		cb.setLineDash(3, 3, 0);
		//cb.setColorStroke(clrDisp);
		
		cb.setLineWidth(0.8f);
		cb.moveTo(10, 20);
		cb.lineTo(594-10, 20);
		cb.stroke();

        cb.saveState();
        cb.restoreState();
        intProgress++;
    }
    
	public void afterPropertiesSet() throws Exception {
		// TODO Auto-generated method stub

	}

	
	public SrcAhd getAssembly() {
		return assembly;
	}

	public void setAssembly(SrcAhd assembly) {
		this.assembly = assembly;
	}

	public List getListPL() {
		return listPL;
	}

	public void setListPL(List listPL) {
		this.listPL = listPL;
	}

	public RegisterdMasterCompany getMasterCompany() {
		return masterCompany;
	}

	public void setMasterCompany(RegisterdMasterCompany masterCompany) {
		this.masterCompany = masterCompany;
	}

	public Project getProject() {
		return project;
	}

	public void setProject(Project project) {
		this.project = project;
	}

	public SystemInfo getSystemInfo() {
		return systemInfo;
	}

	public void setSystemInfo(SystemInfo systemInfo) {
		this.systemInfo = systemInfo;
	}
	
	public String Double2String(Double value) {
		if(value==null) {
			return "0.0";
		}
		
		return decimalFormat.format(value);
	}
}
