package com.rfxsoft.justrfx.document;

import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.PageSize;
import com.lowagie.text.pdf.PdfWriter;
import com.rfxsoft.justrfx.DocuType;
import com.rfxsoft.justrfx.business.b_rtng.RtngManager;
import com.rfxsoft.justrfx.business.d_mtrl.MtrlManager;
import com.rfxsoft.justrfx.business.j_prch.PrchManager;
import com.rfxsoft.justrfx.domain.Project;
import com.rfxsoft.justrfx.domain.RegisterdMasterCompany;
import com.rfxsoft.justrfx.domain.RtgAst;
import com.rfxsoft.justrfx.domain.SrcAhd;
import com.rfxsoft.justrfx.domain.SystemInfo;
import com.rfxsoft.justrfx.domain.XpoBhd;
import com.rfxsoft.justrfx.web.views.RfxPdfDocuLayout;

public class PdfRunnableFormatter implements Runnable {
	
	private static final int OUTPUT_BYTE_ARRAY_INITIAL_SIZE = 512;
	
	private RegisterdMasterCompany masterCompany;
	private SystemInfo	systemInfo;
	private PrchManager purchaseManager;
	private RtngManager routingManager;
	private MtrlManager	materialManager;
	
	
	//PDF Data  .
	private ByteArrayOutputStream baos;
	
	//Ȳ ۼƮ
	private int p;
	private int createdPage;
	private int redrawcount;
	
	//ϸ
	private String fileName;
	//ȣ
	private String rtgast_uid;
	//Ÿ
	private DocuType docuType;
	//POȣ
	private String po_no;
	
	public PdfRunnableFormatter(String file_name, String docu_no, DocuType docu_type, String po_no,
			RegisterdMasterCompany masterCompany,
			SystemInfo	systemInfo,
			RtngManager routingManager,
			PrchManager purchaseManager,
			MtrlManager	materialManager
			) {
		super();
		baos = new ByteArrayOutputStream(OUTPUT_BYTE_ARRAY_INITIAL_SIZE);
		p = 0;
		createdPage=0;
		redrawcount=0;
		
		this.fileName = file_name;
		this.rtgast_uid = docu_no;
		this.docuType = docu_type;
		this.po_no = po_no;
		
		this.masterCompany		= masterCompany;
		this.systemInfo			= systemInfo;
		
		this.purchaseManager	= purchaseManager;
		this.routingManager		= routingManager;
		this.materialManager	= materialManager;
	}

	
	public boolean PdfDocuPoProcess (Document document, PdfWriter writer) {
		
		Integer po_uid = Integer.parseInt(rtgast_uid);
		
		if(po_uid < 0)
			return false;
		
		XpoBhd xpobhd = routingManager.getXpoBhdById(po_uid);
		if(xpobhd==null)
			return false;
		List listXpoAst = purchaseManager.getXpoAstList(po_no);
		if(listXpoAst==null)
			return false;
		
		PdfDocuPo pdfDocu;
		String com_logo = this.systemInfo.getImagePath() + "/" + this.masterCompany.getCompanyLogo();
		String com_sign = this.systemInfo.getImagePath() + "/" + this.masterCompany.getCompanySign();

		pdfDocu =
			new PdfDocuPo(writer, document, createdPage, this.rtgast_uid, com_logo, com_sign,
					this.masterCompany,
					this.systemInfo,
					xpobhd,
					listXpoAst
			);
		
		pdfDocu.DoBuildProcess();
		
		return true;
	}

	
	public boolean PdfDocuAcProcess (Document document, PdfWriter writer) {
		
		Integer pr_uid = Integer.parseInt(rtgast_uid);
		
		if(pr_uid < 0)
			return false;
		
		RtgAst prHeader = routingManager.getRtgAstById(pr_uid);
		if(prHeader==null)
			return false;
		List listXpoast = purchaseManager.getAcDetail(po_no);
		if(listXpoast==null)
			return false;
		
		PdfDocuAc pdfDocu;
		String com_logo = this.systemInfo.getImagePath() + "/" + this.masterCompany.getCompanyLogo();
		String com_sign = this.systemInfo.getImagePath() + "/" + this.masterCompany.getCompanySign();

		pdfDocu =
			new PdfDocuAc(writer, document, createdPage, this.rtgast_uid, com_logo, com_sign,
					this.masterCompany,
					this.systemInfo,
					prHeader,
					listXpoast
			);
		
		pdfDocu.DoBuildProcess();
		
		return true;
	}
	public boolean PdfDocuPrProcess (Document document, PdfWriter writer) {
		
		Integer pr_uid = Integer.parseInt(rtgast_uid);
		
		if(pr_uid < 0)
			return false;
		
		RtgAst prHeader = routingManager.getRtgAstById(pr_uid);
		if(prHeader==null)
			return false;
		List listPL = purchaseManager.getPrDetail(pr_uid);
		if(listPL==null)
			return false;
		
		PdfDocuPr pdfDocu;
		String com_logo = this.systemInfo.getImagePath() + "/" + this.masterCompany.getCompanyLogo();
		String com_sign = this.systemInfo.getImagePath() + "/" + this.masterCompany.getCompanySign();

		pdfDocu =
			new PdfDocuPr(writer, document, createdPage, this.rtgast_uid, com_logo, com_sign,
					this.masterCompany,
					this.systemInfo,
					prHeader,
					listPL
			);
		
		pdfDocu.DoBuildProcess();
		
		return true;
	}

	public boolean PdfDocuPlProcess (Document document, PdfWriter writer) {
		
		Integer assy_uid=0;
		Integer pj_uid=0;
		
		try
		{
			assy_uid = Integer.parseInt(rtgast_uid);
			pj_uid = Integer.parseInt(po_no);
		}
		catch(NumberFormatException e)
		{
			return false;
		}
		
		Project project = materialManager.getProject(pj_uid);
		SrcAhd assembly = materialManager.getMaterialByUnique_id(assy_uid);
		
		if(project==null || assembly==null)
			return false;
		
		List listPL = materialManager.getPartListByParent(assy_uid);
		if(listPL==null)
			return false;
		
		PdfDocuPl pdfDocu;
		String com_logo = this.systemInfo.getImagePath() + "/" + this.masterCompany.getCompanyLogo();
		String com_sign = this.systemInfo.getImagePath() + "/" + this.masterCompany.getCompanySign();

		pdfDocu =
			new PdfDocuPl(writer, document, createdPage, this.rtgast_uid, com_logo, com_sign,
					this.masterCompany,
					this.systemInfo,
					project,
					assembly,
					listPL
			);
		
		pdfDocu.DoBuildProcess();
		
		return true;
	}

	public void run() {
		int header_height = 0;
		
		switch(docuType) {
		case PUR_ORDER:
			header_height = 195;
			break;
		case PUR_REQUEST:
			header_height = 100;
			break;
		case PART_LIST:
			header_height = 90;
			break;
		case AC_SHEET:
			header_height = 90;
			break;
		default:
			header_height = 150;
		}
		
		Document document = newDocument();
		PdfWriter writer=null;
		try {
			writer = PdfWriter.getInstance(document, baos);
		} catch (DocumentException e) {
			e.printStackTrace();
		}
		// Apply preferences and build metadata.
		writer.setViewerPreferences(getViewerPreferences());
		
		document.setMargins(5, 5, header_height, 20);
		document.addTitle("justRFX Document");
		document.addAuthor("RFX Soft");
		document.addSubject("justRFX ۹");
		document.addKeywords("RFX, justRFX, RFXSoft");
		document.addCreator("Programmed by k1park");

		// Build PDF document.
		document.open();
		
		switch(docuType) {
		case PUR_ORDER:
			if( PdfDocuPoProcess (document, writer) == true) {	
				CreateFile(baos, this.fileName);
			}
			break;
		case PUR_REQUEST:
			if( PdfDocuPrProcess (document, writer) == true) {	
				CreateFile(baos, this.fileName);
			}
			break;
		case PART_LIST:
			if( PdfDocuPlProcess (document, writer) == true) {	
				CreateFile(baos, this.fileName);
			}
			break;
		case QUOTATION:
			break;
		case AC_SHEET:
			if( PdfDocuAcProcess (document, writer) == true) {	
				CreateFile(baos, this.fileName);
			}
			break;
		default:
			RfxPdfDocuLayout pdf_layout	= new RfxPdfDocuLayout(writer, document, createdPage, this.rtgast_uid);
			pdf_layout.DoWork();
			CreateFile(baos, this.fileName);
		}
		document.close();
		p = 100;
	}
	
	public void CreateFile(ByteArrayOutputStream bo, String file_name) {

		FileOutputStream f_out=null;
		try {
			f_out = new FileOutputStream(file_name);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} 
		try {
			f_out.write(bo.toByteArray());
		} catch (IOException e) {
			e.printStackTrace();
		} //Ͽ   .
		try {
			f_out.close();
		} catch (IOException e) {
			e.printStackTrace();
		} //ä ݴ´. ̶ Ʈ .

	}

	/**
	 * Create a new document to hold the PDF contents.
	 * <p>By default returns an A4 document, but the subclass can specify any
	 * Document, possibly parameterized via bean properties defined on the View.
	 * @return the newly created iText Document
	 * @see com.lowagie.text.Document#Document(com.lowagie.text.Rectangle)
	 */
	protected Document newDocument() {
		return new Document(PageSize.A4);
	}

	/**
	 * Return the viewer preferences for the PDF file.
	 * <p>By default returns <code>AllowPrinting</code> and
	 * <code>PageLayoutSinglePage</code>, but can be subclassed.
	 * The subclass can either have fixed preferences or retrieve
	 * them from bean properties defined on the View.
	 * @return an int containing the bits information against PdfWriter definitions
	 * @see com.lowagie.text.pdf.PdfWriter#AllowPrinting
	 * @see com.lowagie.text.pdf.PdfWriter#PageLayoutSinglePage
	 */
	protected int getViewerPreferences() {
		return PdfWriter.AllowPrinting | PdfWriter.PageLayoutSinglePage;
	}
	
	/**
	 * Gets the current percentage of the file that is done.
	 * @return a percentage or -1 if something went wrong.
	 */
	public int getPercentage() {
		return p;
	}
	
	/**
	 * Gets the complete PDF data
	 * @return the PDF as an array of bytes
	 * @throws DocumentException when the document isn't ready yet
	 */
	public ByteArrayOutputStream getPdf() throws DocumentException {
		if (p < 100) {
			throw new DocumentException("The document isn't finished yet!");
		}
		return baos;
	}
	
	public int getCreatedPage() {
		return createdPage;
	}
	public int getRedrawCount() {
		redrawcount++;
		return redrawcount-1;
	}
}
